From 423a959a268b205c048489e6addeba5db449e322 Mon Sep 17 00:00:00 2001
From: Yingdong Liu <liuyingdong@huawei.com>
Date: Thu, 15 Dec 2016 09:09:16 +0800
Subject: [PATCH] [PATCH]netfront: need release all resources after adding and
 removing NICs time and again

---
 sys/dev/xen/netfront/netfront.c | 46 +++++++++++++++++++++++++++++++++++++++++
 1 file changed, 46 insertions(+)

diff --git a/sys/dev/xen/netfront/netfront.c b/sys/dev/xen/netfront/netfront.c
index 0caaa2c..5b67e12 100644
--- a/sys/dev/xen/netfront/netfront.c
+++ b/sys/dev/xen/netfront/netfront.c
@@ -2196,6 +2196,46 @@ netfront_detach(device_t dev)
 }
 
 static void
+netif_release_rx_bufs_copy(struct netfront_info *np)
+{
+	struct mbuf *m;
+	int i, ref;
+	int busy = 0, inuse = 0;
+
+	XN_RX_LOCK(np);
+
+	for (i = 0; i < NET_RX_RING_SIZE; i++) {
+		ref = np->grant_rx_ref[i];
+
+		if (ref == GRANT_REF_INVALID)
+			continue;
+
+		inuse++;
+
+		m = np->rx_mbufs[i];
+
+		if (!gnttab_end_foreign_access_ref(ref))
+		{
+			busy++;
+			continue;
+		}
+
+		gnttab_release_grant_reference(&np->gref_rx_head, ref);
+		np->grant_rx_ref[i] = GRANT_REF_INVALID;
+		add_id_to_freelist(np->rx_mbufs, i);
+
+		m_freem(m);
+	}
+
+	if (busy)
+		device_printf(np->xbdev, "Unable to release %d of %d "
+			"inuse grant references out of %ld total.\n",
+			busy, inuse, NET_RX_RING_SIZE);
+	
+	XN_RX_UNLOCK(np);
+}
+
+static void
 netif_free(struct netfront_info *info)
 {
 	XN_LOCK(info);
@@ -2209,6 +2249,12 @@ netif_free(struct netfront_info *info)
 		info->xn_ifp = NULL;
 	}
 	ifmedia_removeall(&info->sc_media);
+	netif_release_tx_bufs(info);
+	if (info->copying_receiver)
+		netif_release_rx_bufs_copy(info);
+
+	gnttab_free_grant_references(info->gref_tx_head);
+	gnttab_free_grant_references(info->gref_rx_head);
 }
 
 static void
-- 
1.8.3.4

